/* $Id: window.cpp 62 2011-03-07 05:58:30Z oh2gve $
 *
 * Copyright 2011 Tapio Aaltonen
 *
 *
 * This file is part of Kello.
 *
 * Kello is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Kello is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Kello.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


// Own stuff.
#include "window.h"
#include "config.h"

// Wx stuff.
#include <wx/frame.h>
#include <wx/string.h>
#include <wx/log.h>
#include <wx/menu.h>
#include <wx/msgdlg.h>
#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/datetime.h>
#include <wx/fontdlg.h>
#include <wx/colordlg.h>



// Event table for events handled in this code file.

BEGIN_EVENT_TABLE(Window, wxFrame)
	EVT_TIMER(ID_Timer, Window::onTimer)
	EVT_MENU(ID_Font, Window::onFont)
	EVT_MENU(ID_BGColor, Window::onBGColor)
	EVT_MENU(ID_Info, Window::onInfo)
	EVT_MENU(ID_Quit, Window::onQuit)
	EVT_CLOSE(Window::onClose)
END_EVENT_TABLE()


Window::Window(wxString const& title):
wxFrame(NULL, -1, title, wxDefaultPosition, wxDefaultSize, wxMINIMIZE_BOX|wxSYSTEM_MENU|wxCAPTION|wxCLOSE_BOX|wxCLIP_CHILDREN), timeField_(NULL), timer_(NULL)
{
	// Create main menu.
	wxMenu* mainMenu = new wxMenu;

	// Create menu items.
	mainMenu->Append(ID_Font, wxT(FONT));
	mainMenu->Append(ID_BGColor, wxT(BGCOLOR));
	mainMenu->Append(ID_Info, wxT(INFO));
	mainMenu->AppendSeparator();
	mainMenu->Append(ID_Quit, wxT(QUIT));

	// Create menubar and add menu to it.
	wxMenuBar* menuBar = new wxMenuBar;
	menuBar->Append(mainMenu, wxT(FUNCTIONS));
	SetMenuBar(menuBar);

	
	// Create text box for time text.
	timeField_ = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_READONLY|wxTE_RICH2|wxTE_CENTRE);
	timeField_->WriteText(wxDateTime::Now().FormatISOTime());

	// Create top level sizer to handle stuff in the window's main area.
	wxBoxSizer* topSizer = new wxBoxSizer(wxVERTICAL);

	topSizer->Add(
		timeField_,
		1, // Vertically stretchable
		wxEXPAND // Horizontally stretchable
	);

	// Put top sizer in the window.
	SetSizer(topSizer);
	
	// Create timer
	timer_ = new wxTimer(this, ID_Timer);
	if ( !timer_->Start(100) )
	{
		wxLogError(wxT(ERROR_TIMER_START));
	}

	// Optimize window size.
	Fit();
}


Window::~Window()
{
}


// Event handlers.


void Window::onTimer(wxTimerEvent& event)
{
	// Construct new time string.
	wxString newtext = wxDateTime::Now().FormatISOTime();

	// Get current one.
	wxString current = timeField_->GetValue();

	// Replace parts which have changed.
	if ( newtext.substr(0,3) != current.substr(0,3) ) timeField_->Replace(0, 3, newtext.substr(0, 3));
	if ( newtext.substr(3,3) != current.substr(3,3) ) timeField_->Replace(3, 6, newtext.substr(3, 3));
	if ( newtext.substr(6,2) != current.substr(6,2) ) timeField_->Replace(6, 8, newtext.substr(6, 2));
}


void Window::onFont(wxCommandEvent& event)
{
	// Get current style.
	wxTextAttr style = timeField_->GetDefaultStyle();
	wxFontData fontdata;
	fontdata.SetInitialFont(style.GetFont());
	fontdata.SetColour(style.GetTextColour());

	// Update font data.
	wxFontDialog chooser(this, fontdata);
	if ( chooser.ShowModal() == wxID_OK )
	{
		// We got data for new style, set it.
		style.SetFont(chooser.GetFontData().GetChosenFont());
		style.SetTextColour(chooser.GetFontData().GetColour());

		// Update time box style.
		if ( !timeField_->SetDefaultStyle(style) )
		{
			wxLogError(wxT(ERROR_STYLE_SET));
		}

		// Check how much space a max-width text takes with new style.
		int width, height;
		wxFont tmpfont = chooser.GetFontData().GetChosenFont();
		timeField_->GetTextExtent(wxT("88:88:88"), &width, &height, NULL, NULL, &tmpfont);

		// Add some margin.
		width += 6;

		// Set new min size of the box and refit the window.
		timeField_->SetMinSize(wxSize(width, height));
		Fit();

		// Print whole text with new setup.
		wxString val = timeField_->GetValue();
		timeField_->Clear();
		timeField_->WriteText(val);
	}
}


void Window::onBGColor(wxCommandEvent& event)
{
	// Get current style.
	wxTextAttr style = timeField_->GetDefaultStyle();
	wxColourData coldata;
	coldata.SetColour(style.GetBackgroundColour());

	// Pick new one.
	wxColourDialog chooser(this, &coldata);
	if ( chooser.ShowModal() == wxID_OK )
	{
		// We got data for new style, set it.
		style.SetBackgroundColour(chooser.GetColourData().GetColour());

		// Update time box style.
		if ( !timeField_->SetDefaultStyle(style) )
		{
			wxLogError(wxT(ERROR_STYLE_SET));
		}

		// Print whole text with new setup.
		wxString val = timeField_->GetValue();
		timeField_->Clear();
		timeField_->WriteText(val);
	}
}


void Window::onInfo(wxCommandEvent& event)
{
	wxString title = wxT(PACKAGE);
	title += wxT(INFO);
	
	wxString info = wxT(PACKAGE);
	info += wxT(" ");
	info += wxT(VERSION);
	info += wxT(" by ");
	info += wxT(AUTHOR);
	
	wxMessageDialog box (this, info, title, wxOK | wxICON_INFORMATION );
	box.ShowModal();
}


void Window::onQuit(wxCommandEvent& event)
{
	Close(true);
}


void Window::onClose(wxCloseEvent& event)
{
	timer_->Stop();
	Destroy();
}
